/* 
 * Newsletter2Go-API (https://api.newsletter2go.com)
 *
 * <h5>JSON first</h5><br/>Our REST API exchanges data in the JSON data format. Every parameter you pass (with a few exceptions e.g. when you upload files) should therefore be formatted in JSON and our API will always return results in JSON as well.<br/><br/><h5>Very RESTful</h5><br/>Our API follows a very RESTful approach.<br/>Most importantly, we implemented the following four request methods for CRUD operations:<br/><br/>POST - Create a new record<br/>GET - Retrieve / read records without changing anything<br/>PATCH - Update an existing record<br/>DELETE - Delete one or more records<br/><br/><h5>HTTP Status codes</h5><br/>We also follow the most common HTTP status codes when outputting the API response:<br/><br/><b>2xx - Successful calls</b><br/>200 - Success<br/>201 - Created<br/><br/><b>4xx - Error codes</b><br/>400 - API error - inspect the body for detailed information about what went wrong. The most common error is \"code\":1062, which means, that a unique constraint was hit. For example if the name of the group is already in use.<br/>401 - Not authenticated - invalid access_token. Check if the access_token has expired or is incorrect.<br/>403 - Access denied<br/>404 - Call not available or no records found<br/><br/><b>5xx - API error - please contact support</b><br/><br/><h5>Response format</h5><br/>The API always returns data in the following format<br/><br/><code>{<br/>&nbsp;&nbsp;\"info\": {<br/>&nbsp;&nbsp;&nbsp;&nbsp;\"count\": 0<br/>&nbsp;&nbsp;},<br/>&nbsp;&nbsp;\"value\": [<br/>&nbsp;&nbsp;&nbsp;{<br/>&nbsp;&nbsp;&nbsp;&nbsp;\"key\": \"value\"<br/>&nbsp;&nbsp;&nbsp;}<br/>&nbsp;&nbsp;]<br/>}</code><br/><br/><h5>Recurring GET parameters</h5><br/><ul><li>_filter - a complex filter for filtering the result set based on <a target=\"blank\" href=\"https://tools.ietf.org/html/draft-nottingham-atompub-fiql-00\">FIQL</a>.</li><li>_limit - the maximum number records returned.</li><li>_offset - pagination for the result-set</li><li>_expand - submit true to get all default fields of the resource</li><li>_fields - submit a comma-separated list of case-sensetive field-names to get the values of these fields in the response. You can use this the get values of custom attribute of recipients for example.</li></ul><br/><br/><h5>Filter language</h5><br/>The filter language for filtering results is based on <a target=\"blank\" href=\"https://tools.ietf.org/html/draft-nottingham-atompub-fiql-00\">FIQL</a>.<br/>With the only restriction, that plain values must be surrounded by \". For example first_name==\"Max\"<br/>The following operators are supported<ul><li>Equals - <b>==</b></li><li>Not equals - <b>=ne=</b></li><li>Greater than - <b>=gt=</b></li><li>Greater than equals - <b>=ge=</b></li><li>Lower than - <b>=lt=</b></li><li>Lower than equals - <b>=le=</b></li><li>Like - <b>=like=</b> (in combination with % you are able to search for <i>starts with</i>, <i>ends with</i>, <i>contains</i>. For example first_name=like=\"%Max%\")</li><li>Not like - <b>=nlike=</b></li><li>Logical and - <b>;</b></li><li>Logical or - <b>,</b></li></ul><br/><br/><h5>How to make Calls?</h5><br/>After you authenticated and received a valid access_token, you have to pass it in every consecutive call. Use the Authorization header for that purpose as follows:<br/><code>xhr.setRequestHeader(\"Authorization\", \"Bearer \" + your_access_token);</code><br/><br/>Every call takes additional parameters that can be used to modify the request. These parameters should be passed as JSON<br/><code>var params = {<br/>&nbsp;&nbsp;\"key\"= \"value\"<br/>}<br/>xhr.send(JSON.stringify(params));</code><br/><br/><h5>Sending transactional emails</h5><br/>It is very important to understand the following concept in order for you to take full advantage of our powerful personalization features and our detailed reports when sending transactional emails.<br/><br/>First, you will have to create a new mailing. We recommend that you create that mailing through our UI in order to take full advantage of our powerful newsletter builder. This way, we will automatically create cross-client optimized and responsive HTML. Yet another advantage lies in the possibility for other users (e.g. the marketing team) to change the layout or the content of the mailing without having to contact the developer (you).<br/><br/>Of course, it is also possible to create a mailing through the API. When doing so, you can also take advantage of our cross-client optimized responsive HTML by passing us JSON or YAML according to the Newsletter2Go Markup Language.<br/>No matter how you create the mailing, try to create *one* reusable template. You can customize individual emails by inserting placeholders for personalized fields such as name, products or other information that will be filled through an API call when sending.<br/><br/>By only creating one template, you can take advantage of our full reporting since all emails will be treated part of a \"campaign\". When you change that template, we will create a new version of the mailing in the background and you will be able to see the difference in performance in the reports. This is particularily useful when you are trying to test and optimize different versions of transactional emails such as a sign up email.<br/><br/>After creating a mailing, you will have access to its ID. Use that ID to actually send the email in the next step.<br/><br/>When sending an email, you have to pass the newsletter ID and information about the recipient. Either pass the recipient ID or pass all the recipient's data (including the e-mail-address) as JSON.<br/><br/>If you only pass the recipient ID, we will use his or her data from your list to personalize the mailing. If you pass full recipient data as JSON, we will try to merge the data with existing data from your list.<br/><br/>You can also pass additional data such as product data which is not saved in your list. Just make sure that the placeholders in the source of the mailing correspond to the parameters that you are passing.<br/>This way you can also create for-loops which can be useful if you pass different amounts of data for each recipient (e.g. a purchase confirmation where you want to list all the products that were just bought).
 *
 * OpenAPI spec version: 1.0.0
 * Contact: support@newsletter2go.com
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


using NUnit.Framework;

using System;
using System.Linq;
using System.IO;
using System.Collections.Generic;
using IO.Swagger.Api;
using IO.Swagger.Model;
using IO.Swagger.Client;
using System.Reflection;

namespace IO.Swagger.Test
{
    /// <summary>
    ///  Class for testing RecipientGet
    /// </summary>
    /// <remarks>
    /// This file is automatically generated by Swagger Codegen.
    /// Please update the test case below to test the model.
    /// </remarks>
    [TestFixture]
    public class RecipientGetTests
    {
        private RecipientGet instance;

        /// <summary>
        /// Setup before each test
        /// </summary>
        [SetUp]
        public void Init()
        {
            instance = new RecipientGet();
        }

        /// <summary>
        /// Clean up after each test
        /// </summary>
        [TearDown]
        public void Cleanup()
        {

        }

        /// <summary>
        /// Test an instance of RecipientGet
        /// </summary>
        [Test]
        public void RecipientGetInstanceTest()
        {
            Assert.IsInstanceOf<RecipientGet> (instance, "instance is a RecipientGet");
        }

        /// <summary>
        /// Test the property 'Id'
        /// </summary>
        [Test]
        public void IdTest()
        {
            // TODO: unit test for the property 'Id'
        }
        /// <summary>
        /// Test the property 'Email'
        /// </summary>
        [Test]
        public void EmailTest()
        {
            // TODO: unit test for the property 'Email'
        }
        /// <summary>
        /// Test the property 'Phone'
        /// </summary>
        [Test]
        public void PhoneTest()
        {
            // TODO: unit test for the property 'Phone'
        }
        /// <summary>
        /// Test the property 'Gender'
        /// </summary>
        [Test]
        public void GenderTest()
        {
            // TODO: unit test for the property 'Gender'
        }
        /// <summary>
        /// Test the property 'FirstName'
        /// </summary>
        [Test]
        public void FirstNameTest()
        {
            // TODO: unit test for the property 'FirstName'
        }
        /// <summary>
        /// Test the property 'LastName'
        /// </summary>
        [Test]
        public void LastNameTest()
        {
            // TODO: unit test for the property 'LastName'
        }
        /// <summary>
        /// Test the property 'IsUnsubscribed'
        /// </summary>
        [Test]
        public void IsUnsubscribedTest()
        {
            // TODO: unit test for the property 'IsUnsubscribed'
        }
        /// <summary>
        /// Test the property 'IsBlacklisted'
        /// </summary>
        [Test]
        public void IsBlacklistedTest()
        {
            // TODO: unit test for the property 'IsBlacklisted'
        }
        /// <summary>
        /// Test the property 'CustomAttribute_'
        /// </summary>
        [Test]
        public void CustomAttribute_Test()
        {
            // TODO: unit test for the property 'CustomAttribute_'
        }
        /// <summary>
        /// Test the property 'Rating'
        /// </summary>
        [Test]
        public void RatingTest()
        {
            // TODO: unit test for the property 'Rating'
        }
        /// <summary>
        /// Test the property 'StatisticMailsReceived'
        /// </summary>
        [Test]
        public void StatisticMailsReceivedTest()
        {
            // TODO: unit test for the property 'StatisticMailsReceived'
        }
        /// <summary>
        /// Test the property 'StatisticMailsOpened'
        /// </summary>
        [Test]
        public void StatisticMailsOpenedTest()
        {
            // TODO: unit test for the property 'StatisticMailsOpened'
        }
        /// <summary>
        /// Test the property 'StatisticMailsUniqueOpened'
        /// </summary>
        [Test]
        public void StatisticMailsUniqueOpenedTest()
        {
            // TODO: unit test for the property 'StatisticMailsUniqueOpened'
        }
        /// <summary>
        /// Test the property 'StatisticMailsClicked'
        /// </summary>
        [Test]
        public void StatisticMailsClickedTest()
        {
            // TODO: unit test for the property 'StatisticMailsClicked'
        }
        /// <summary>
        /// Test the property 'StatisticConversions'
        /// </summary>
        [Test]
        public void StatisticConversionsTest()
        {
            // TODO: unit test for the property 'StatisticConversions'
        }
        /// <summary>
        /// Test the property 'StatisticConversionsSum'
        /// </summary>
        [Test]
        public void StatisticConversionsSumTest()
        {
            // TODO: unit test for the property 'StatisticConversionsSum'
        }
        /// <summary>
        /// Test the property 'StatisticLastMailAction'
        /// </summary>
        [Test]
        public void StatisticLastMailActionTest()
        {
            // TODO: unit test for the property 'StatisticLastMailAction'
        }
        /// <summary>
        /// Test the property 'StatisticLastMailReceived'
        /// </summary>
        [Test]
        public void StatisticLastMailReceivedTest()
        {
            // TODO: unit test for the property 'StatisticLastMailReceived'
        }
        /// <summary>
        /// Test the property 'StatisticLastMailOpened'
        /// </summary>
        [Test]
        public void StatisticLastMailOpenedTest()
        {
            // TODO: unit test for the property 'StatisticLastMailOpened'
        }
        /// <summary>
        /// Test the property 'StatisticLastMailClicked'
        /// </summary>
        [Test]
        public void StatisticLastMailClickedTest()
        {
            // TODO: unit test for the property 'StatisticLastMailClicked'
        }
        /// <summary>
        /// Test the property 'StatisticLastMailConverted'
        /// </summary>
        [Test]
        public void StatisticLastMailConvertedTest()
        {
            // TODO: unit test for the property 'StatisticLastMailConverted'
        }
        /// <summary>
        /// Test the property 'StatisticLastUseragentDevice'
        /// </summary>
        [Test]
        public void StatisticLastUseragentDeviceTest()
        {
            // TODO: unit test for the property 'StatisticLastUseragentDevice'
        }
        /// <summary>
        /// Test the property 'StatisticLastUseragentFamily'
        /// </summary>
        [Test]
        public void StatisticLastUseragentFamilyTest()
        {
            // TODO: unit test for the property 'StatisticLastUseragentFamily'
        }
        /// <summary>
        /// Test the property 'StatisticLastLatitude'
        /// </summary>
        [Test]
        public void StatisticLastLatitudeTest()
        {
            // TODO: unit test for the property 'StatisticLastLatitude'
        }
        /// <summary>
        /// Test the property 'StatisticLastLongitude'
        /// </summary>
        [Test]
        public void StatisticLastLongitudeTest()
        {
            // TODO: unit test for the property 'StatisticLastLongitude'
        }
        /// <summary>
        /// Test the property 'StatisticLastBlacklistList'
        /// </summary>
        [Test]
        public void StatisticLastBlacklistListTest()
        {
            // TODO: unit test for the property 'StatisticLastBlacklistList'
        }
        /// <summary>
        /// Test the property 'StatisticLastUnsubscribeList'
        /// </summary>
        [Test]
        public void StatisticLastUnsubscribeListTest()
        {
            // TODO: unit test for the property 'StatisticLastUnsubscribeList'
        }
        /// <summary>
        /// Test the property 'StatisticLastBlacklistListReason'
        /// </summary>
        [Test]
        public void StatisticLastBlacklistListReasonTest()
        {
            // TODO: unit test for the property 'StatisticLastBlacklistListReason'
        }
        /// <summary>
        /// Test the property 'StatisticLastUnsubscribeListReason'
        /// </summary>
        [Test]
        public void StatisticLastUnsubscribeListReasonTest()
        {
            // TODO: unit test for the property 'StatisticLastUnsubscribeListReason'
        }

    }

}
